package com.contextspace;

import static org.junit.Assert.*;

import java.awt.print.Printable;
import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.List;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.contextspace.objectdb_example.dao.IAttributeNameDAO;
import com.contextspace.objectdb_example.dao.IHolderDAO;
import com.contextspace.objectdb_example.dao.IOtherObjectDAO;
import com.contextspace.objectdb_example.model.AttributeName;
import com.contextspace.objectdb_example.model.OtherObject;
import com.contextspace.objectdb_example.model.holder.Holder;

public class ExampleTest {

    private static ClassPathXmlApplicationContext context = null;
    
    // attribute
    private static IAttributeNameDAO attrNameDao;
    private static IHolderDAO holderDao;
    private static IOtherObjectDAO otherObjectDao;
    
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        context = new ClassPathXmlApplicationContext(new String[] { "applicationContext-test.xml" });
        
        attrNameDao = context.getBean(IAttributeNameDAO.class);
        holderDao = context.getBean(IHolderDAO.class);
        otherObjectDao = context.getBean(IOtherObjectDAO.class);
    }
    
    @AfterClass
    public static void tearDownAfterClass() throws Exception {
        if(context != null) {
            context.close();
        }      
        
        File f1 = new File("objectdb"+File.separator+"db");
        deleteDir(f1);
    }
    
    
    @Test
    public void test() throws Exception {
        
        assertEquals(0, attrNameDao.count());
        
        AttributeName an1 = new AttributeName("an1", "an1 eng");
        AttributeName an2 = new AttributeName("an2", "an2 eng");
        an2.setDisplayValue("fr", "an2 fr");
        AttributeName an3 = new AttributeName("an3", "an3 eng");
        an3.setDisplayValue("fr", "an3 fr");
        
        an1 = attrNameDao.create(an1);
        an2 = attrNameDao.create(an2);
        an3 = attrNameDao.create(an3);
        
        assertEquals(3, attrNameDao.count());
        
        assertEquals("<attributename name=\"an1\" values=\"en=an1 eng\" id=\"1\"/>", attrNameDao.find(an1.getId()).toString());
        assertEquals("<attributename name=\"an2\" values=\"fr=an2 fr,en=an2 eng\" id=\"2\"/>", attrNameDao.find(an2.getId()).toString());
        assertEquals("<attributename name=\"an3\" values=\"fr=an3 fr,en=an3 eng\" id=\"3\"/>", attrNameDao.find(an3.getId()).toString());
        
        
        Holder holder = new Holder();
        holder = holderDao.create(holder);
        assertEquals("<holder id=\"4\">null</holder>", holderDao.find(holder.getId()).toString());
        
        
        Holder holder2 = new Holder();
        holder2.addName(an1);
        
        holder2 = holderDao.create(holder2);
        
        //get it back using a full select... THIS WORKS
        assertEquals("<holder id=\"5\">null<list><attributename name=\"an1\" values=\"en=an1 eng\" id=\"1\"/></list></holder>", holderDao.lookupById(holder2.getId()).toString());
        
        //get it back using a find... THIS FAILS
        assertEquals("<holder id=\"5\">null<list><attributename name=\"an1\" values=\"en=an1 eng\" id=\"1\"/></list></holder>", holderDao.find(holder2.getId()).toString());

        holderDao.delete(holder);
        holderDao.delete(holder2);
        assertEquals(0, holderDao.count());
        
        attrNameDao.delete(an1);
        attrNameDao.delete(an2);
        attrNameDao.delete(an3);
        
        assertEquals(0, attrNameDao.count());
    }

    
    @Test
    public void test2() throws Exception {
        
        AttributeName an1 = new AttributeName("holdername", "Holder Name");
        AttributeName an2 = new AttributeName("listname1", "List Name1");
        AttributeName an3 = new AttributeName("listname2", "List Name2");
        AttributeName an4 = new AttributeName("listname3", "List Name3");
        
        AttributeName an5 = new AttributeName("mapentry1", "Map Entry1");
        AttributeName an6 = new AttributeName("mapentry2", "Map Entry2");
        
        an1 = attrNameDao.create(an1);
        an2 = attrNameDao.create(an2);
        an3 = attrNameDao.create(an3);
        an4 = attrNameDao.create(an4);
        an5 = attrNameDao.create(an5);
        an6 = attrNameDao.create(an6);
        
        OtherObject oobj1 = new OtherObject("mapentry1 otherobject1");
        OtherObject oobj2 = new OtherObject("mapentry1 otherobject2");
        OtherObject oobj3 = new OtherObject("mapentry2 otherobject1");
        OtherObject oobj4 = new OtherObject("mapentry2 otherobject2");
        
        oobj1 = otherObjectDao.create(oobj1);
        oobj2 = otherObjectDao.create(oobj2);
        oobj3 = otherObjectDao.create(oobj3);
        oobj4 = otherObjectDao.create(oobj4);
        
        List<OtherObject> mapentry1OtherObjects = new ArrayList<OtherObject>();
        mapentry1OtherObjects.add(oobj1);
        mapentry1OtherObjects.add(oobj2);
        
        List<OtherObject> mapentry2OtherObjects = new ArrayList<OtherObject>();
        mapentry2OtherObjects.add(oobj3);
        mapentry2OtherObjects.add(oobj4);
        
        
        assertEquals(0, holderDao.count());
        
        Holder holder = new Holder();
        holder.setName(an1);
        holder.addName(an2);
        holder.addName(an3);
        holder.addName(an4);
        holder.addMapEntry(an5, mapentry1OtherObjects);
        holder.addMapEntry(an6, mapentry2OtherObjects);
        
        System.out.println("BEFORE: " + printPrettyXml(holder.toString()) + "\n\n");
        
        holder = holderDao.create(holder);
        
        holderDao.flush();
        
        holder = holderDao.find(holder.getId());
        System.out.println("AFTER: " + printPrettyXml(holder.toString()));
    }
    
    private static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
    
        // The directory is now empty so delete it
        return dir.delete();
    } 
    
    public static String printPrettyXml(String input) {
        return printPrettyXml(input, 4);
    }
    
    public static String printPrettyXml(String input, int indent) {
        try {
            Source xmlInput = new StreamSource(new StringReader(input));
            StringWriter stringWriter = new StringWriter();
            StreamResult xmlOutput = new StreamResult(stringWriter);
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", String.valueOf(indent));
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.transform(xmlInput, xmlOutput);
            return xmlOutput.getWriter().toString();
        }
        catch(TransformerException e) {
            return "Exception thrown during transform of xml: " + e.getMessage();
        }
    }
}
